const _ = require('lodash');
const path = require('path');
const chalk = require('chalk');
const fs = require("fs");

//pc 和 m 端代码库名称
const REPOS = {
    'umu_pc': 'pc',
    'PC': 'pc',

    'umu_wap_student': 'm',
    'M': 'm',

    // dev
    'test-gen': 'm',
}

const COMPONENT_TYPE_MAP = {
    'SHARE': 'SHARE',
    'PRIVATE': 'PRIVATE'
}

const COMPONENT_TYPES = [{
    name: '共用',
    value: 'SHARE'
}, {
    name: '单个路由页面使用',
    value: 'PRIVATE'
}];

let getUserName = (ctx) => {
    return ctx.user.git.name() ||
        process.env.USER || '';
}

/**
 * 检测当前目录是否在 m 或 pc
 */
let checkCwd = (ctx) => {
    let cwd = path.basename(process.cwd());
    let repoType = REPOS[cwd]; // PC,M
    if (Object.keys(REPOS).indexOf(cwd) > -1) {
        ctx.log(`侦测到当前目录为 ${chalk.yellow(cwd)}，将使用 ${chalk.yellow(REPOS[cwd])} 端模板生成相关文件`);
        return repoType;
    } else {
        ctx.log(`${chalk.red('侦测到当前目录不在 PC 或 M 端代码仓库根目录，程序将在1秒后退出')}`);
        var done = ctx.async();
        setTimeout(() => {
            process.exit(1);
        }, 1000);
    }
}

/**
 * 替换路径中指定的变量
 *
 * @param {any} match 搜索到的变量
 * @param {any} options 生成器的参数
 * @returns 生成器参数中对应的变量值
 */
let replacer = (match, options) => {
    return options[match];
}

/**
 * 判断指定的页面是否存在，
 * 正常情况下页面在项目中的位置为 src/page/${pageName}
 * @param {object} fs yoman fs
 * @param {string} pageName 页面名
 * @returns {bool} 是否存在
 */
let isPageExists = (fs, repoType, pageName) => {
    return fs.exists(`src/page/${_.camelCase(pageName)}/${repoType == REPOS.PC ? _.camelCase(pageName) + '.dwt' : _.snakeCase(pageName) + '.php.dwt'}`);
}


/**
 * 判断指定的路由页面是否存在，
 * 正常情况下路由页面在项目中的位置为 src/page/${pageName}/routePage/${routePageName}
 * @param {object} fs yoman fs
 * @param {string} pageName 页面名
 * @param {string} routePageName 页面名
 * @returns {bool} 是否存在
 */
let isRoutePageExists = (fs, repoType, pageName, routePageName) => {
    return fs.exists('src/page/'
        + _.camelCase(pageName)
        + '/routePage/'
        + _.camelCase(routePageName)
        + '/'
        + _.camelCase(routePageName)
        + repoType == REPOS.PC ? '.js' : '.es6.js');
}

/**
 * 判断指定的组件是否存在，
 * 正常情况下组件在项目中的位置为 src/page/${pageName}/components/${componentName}
 * @param {object} fs yoman fs
 * @param {string} pageName 组件名
 * @returns {bool} 是否存在
 */
let isComponentExists = (fs, repoType, pageName, componentName) => {
    return fs.exists('src/page/'
        + _.camelCase(pageName)
        + '/components/'
        + _.camelCase(componentName)
        + '/'
        + _.camelCase(componentName)
        + repoType == REPOS.PC ? '.js' : '.es6.js');
}

let getDate = () => {
    return new Date().toISOString().substring(0, 10);
}

/**
 * list directories
 * @param {object} yeoman ctx
 * @param {*} path path to list from
 * @returns {array} array of dir names
 */
let expandDir = (path) => {
    try {
        return fs.readdirSync(path);
    } catch (error) {
        console.log(JSON.stringify(error));
        process.exit(1);
    }
}

module.exports = {
    REPOS: REPOS,
    COMPONENT_TYPES: COMPONENT_TYPES,
    COMPONENT_TYPE_MAP: COMPONENT_TYPE_MAP,
    getUserName: getUserName,
    checkCwd: checkCwd,
    replacer: replacer,
    isPageExists: isPageExists,
    isRoutePageExists: isRoutePageExists,
    isComponentExists: isComponentExists,
    getDate: getDate,
    expandDir: expandDir
}
